package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentProductionResult;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentProductionResultRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 自动化线接口获取生产数据
 * @Author : jason.niu
 **/
@Slf4j
@Service("mesAutoLineProductionResultStepService")
public class MesAutoLineProductionResultStepService extends BaseStepService {
    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private MesEquipmentProductionResultRepository equipmentProductionResultRDao;

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();
        final String organizeCode = reqBean.getOrganizeCode();

        //获取工位当前设备信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.dispatchCurCellEquipment(reqBean);

        //配置错误 抛出异常
        if (!productionProcessContext.getSuccess()) {
            stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), productionProcessContext.getMessage());
        }

        List<MesEquipmentProductionResult> productionResults = getEquipmentProductionResult(organizeCode);
        if (CollectionUtils.isEmpty(productionResults)) {
            stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "未获取到设备生产结果!");
        }
        List<MesEquipVariableCollectContext> equipVariableCollectContextList = new ArrayList<>();
        for (MesEquipmentProductionResult result : productionResults) {
            MesEquipVariableCollectContext context = new MesEquipVariableCollectContext(organizeCode, result.getProductSn(), MesExtEnumUtil.CELL_MESSAGE_SOURCE.READ.getValue());
            context.setQty(result.getQty());
            context.setEquipVariableId(result.getId());
            equipVariableCollectContextList.add(context);
        }
        productionDispatchContextStepService.dispatchScanProductSnContext(reqBean, equipVariableCollectContextList);
        return stepSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "收集自动化线生产结果成功!");
    }

    private List<MesEquipmentProductionResult> getEquipmentProductionResult(String organizeCode) {
        List<MesEquipmentProductionResult> result = new ArrayList<>();
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumNOEqualPack(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(), "generateRecord", packBean);
        return equipmentProductionResultRDao.findByHqlWhere(packBean);
    }
}
