package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesEquipmentLogExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.*;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description : 统计每摸模自动喷胶量
 * @Author : jason.niu
 **/
@Slf4j
@Service("mesGlueSprayedAmountStepService")
public class MesGlueSprayedAmountStepService extends BaseStepService {
    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;
    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;
    @Autowired
    private IMesEquipmentLogExtService equipmentLogExtService;
    @Autowired
    private MesEquipmentVariableParamRepository equipVarParamRDao;
    @Autowired
    private MesRawMixCfgDetailRepository rawMixCfgDetailRDao;
    @Autowired
    private MesEquipmentVariableParamLogRepository equipVarParamLogRDao;
    @Autowired
    private MesPartRepository mesPartRDao;
    @Autowired
    private MesProductionRecordRepository productionRecordRDao;

    @Override
    public StepResult execute(StationRequestBean reqBean) {
        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();
        final String organizeCode = reqBean.getOrganizeCode();

        //获取工位当前设备信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.dispatchCurCellEquipment(reqBean);

        //配置错误 抛出异常
        if (!productionProcessContext.getSuccess()) {
            stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), productionProcessContext.getMessage());
        }

        //获取上下文产品加工规则数据信息集合
        List<MesProdRuleContext> prodRuleContextList = productionDispatchContextStepService.getProdRuleDataContext(reqBean);
        if (CollectionUtils.isEmpty(prodRuleContextList)) {
            stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "当前不存在产品加工规则信息,请重置工序解决!");
        }
        Map<Integer, List<MesProdRuleContext>> prodRuleContextMap = prodRuleContextList.stream().collect(Collectors.groupingBy(MesProdRuleContext::getForeignKey));

        //当前工位使用的设备
        MesCellEquipContext cellEquipContext = productionProcessContext.getCurCellEquip();
        Map<Long, String> equipVariableIds = getEquipmentVariable(organizeCode, cellEquipContext.getEquipId());
        if (CollectionUtils.isEmpty(equipVariableIds)) {
            stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), String.format("设备ID[%s]未匹配到设备变量参数,请检查数据!", cellEquipContext.getEquipId()));
        }
        List<MesEquipVariableCollectContext> variableContexts = equipmentLogExtService.getEquipmentLogCollectList(organizeCode, cellEquipContext.getEquipId(), MesExtEnumUtil.EQUIP_VARIABLE_NEED_NEW_VALUE.FALSE.getValue(), MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PROCESS_FINISH.getValue(), new ArrayList<>(equipVariableIds.keySet()));
        if (CollectionUtils.isEmpty(variableContexts)) {
            stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), String.format("设备ID[%s]未获取到设备数据,请检查数据!", cellEquipContext.getEquipId()));
        }
        Map<String, String> variableMap = new HashMap<>();
        for (MesEquipVariableCollectContext context : variableContexts) {
            String variableParamType = equipVariableIds.get(context.getEquipVariableId());
            if (!StringUtils.isEmpty(variableParamType)) {
                variableMap.put(variableParamType, context.getEquipVariableValue());
            }
        }

        //1. 获取上下文中生成的主条码
        List<MesProductionPsOutContext> productionPsOutContextList = productionDispatchContextStepService.getProductionPsOutContext(reqBean);
        for (MesProductionPsOutContext sn : productionPsOutContextList) {
            List<MesProdRuleContext> mesProdRuleContexts = prodRuleContextMap.get(sn.getForeignKey());
            MesProdRuleContext prodRuleContext = CollectionUtils.isEmpty(mesProdRuleContexts) ? null : mesProdRuleContexts.get(0);
            if (null == prodRuleContext || StringUtils.isEmpty(prodRuleContext.getAssemblyDataJson())) {
                stepExpSendMsgAndThrowEx(reqBean, resultBean, String.format("零件号[%s]的加工规则丢失,请配置!", sn.getPartNo()));
            }

            //获取非排序装配件清单
            List<MesProductionAssemblyNosortContext> productionAssemblyNosortContextList = prodRuleContext.getNosortAssemblyDataContext();

            MesEquipmentVariableParamLog paramLog = new MesEquipmentVariableParamLog();
            paramLog.setOrganizeCode(reqBean.getOrganizeCode());
            paramLog.setPartNo(prodRuleContext.getOutPartNo());
            MesPart mesPart = mesPartRDao.getByProperty(new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.PART_NO},
                    new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), prodRuleContext.getOutPartNo()});
            if (mesPart != null) {
                paramLog.setPartName(mesPart.getPartName());
            }
            paramLog.setProductSn(sn.getProductSn());
            paramLog.setWorkCellCode(reqBean.getWorkCellCode());
            paramLog.setWorkCenterCode(reqBean.getWorkCenterCode());
            paramLog.setEquipId(cellEquipContext.getEquipId());
            paramLog.setEquipmentCode(cellEquipContext.getEquipmentCode());
            paramLog.setEquipmentName(cellEquipContext.getEquipmentName());
            if (prodRuleContext.getProductionRecordId() != null) {
                MesProductionRecord productionRecord = productionRecordRDao.getById(prodRuleContext.getProductionRecordId());
                if (productionRecord != null) {
                    paramLog.setStartDateTime(productionRecord.getStartDateTime());
                    paramLog.setCompleteDateTime(productionRecord.getCompleteDateTime());
                }
            }
            //遍历装配件清单
            for (MesProductionAssemblyNosortContext productionAssemblyNosortContext : productionAssemblyNosortContextList) {
                String assemblyPartNo = productionAssemblyNosortContext.getAssemblyPartNo();
                MesRawMixCfgDetail detail = rawMixCfgDetailRDao.getByProperty(
                        new String[]{"organizeCode", "isValid", "isDeleted", "outPartNo", "partNo"},
                        new Object[]{organizeCode, CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue(), prodRuleContext.getOutPartNo(), productionAssemblyNosortContext.getAssemblyPartNo()});
                if (detail == null) {
                    continue;
                }
                Integer rawType = detail.getRawType();
                if (Objects.equals(rawType, MesExtEnumUtil.RAW_MIX_DETAIL_TYPE.RAW_MIX_DETAIL_TYPE_10.getValue())) {
                    paramLog.setGluePartNo(assemblyPartNo);
                } else if (Objects.equals(rawType, MesExtEnumUtil.RAW_MIX_DETAIL_TYPE.RAW_MIX_DETAIL_TYPE_20.getValue())) {
                    paramLog.setCuringPartNo(assemblyPartNo);
                } else if (Objects.equals(rawType, MesExtEnumUtil.RAW_MIX_DETAIL_TYPE.RAW_MIX_DETAIL_TYPE_30.getValue())) {
                    paramLog.setAdhesivePartNo(assemblyPartNo);
                } else if (Objects.equals(rawType, MesExtEnumUtil.RAW_MIX_DETAIL_TYPE.RAW_MIX_DETAIL_TYPE_40.getValue())) {
                    paramLog.setInkPartNo(assemblyPartNo);
                } else if (Objects.equals(rawType, MesExtEnumUtil.RAW_MIX_DETAIL_TYPE.RAW_MIX_DETAIL_TYPE_50.getValue())) {
                    paramLog.setDiluentPartNo(assemblyPartNo);
                }
            }
            paramLog.setParamValue(JSONUtil.toJsonStr(variableMap));
            ConvertBean.serviceModelInitialize(paramLog, reqBean.getUserInfo());
            equipVarParamLogRDao.insert(paramLog);
        }
        return stepSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "统计每模自动喷胶量!");
    }

    // 获取设备变量ID对应的变量参数类型
    private Map<Long, String> getEquipmentVariable(String organizeCode, Integer equipId) {
        Map<Long, String> result = new HashMap<>();
        if (equipId != null) {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getNumEqualPack(equipId, "equipId", packBean);
            DdlPreparedPack.getIsNotNull("equipVariableId", packBean);
            DdlPreparedPack.getIsNotNull("paramType", packBean);
            List<MesEquipmentVariableParam> variableParams = equipVarParamRDao.findByHqlWhere(packBean);
            for (MesEquipmentVariableParam param : variableParams) {
                result.put(param.getEquipVariableId().longValue(), param.getParamType());
            }
        }
        return result;
    }
}
