package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesStationContainerSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerPackage;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerPackageDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerSnStation;
import cn.estsh.i3plus.pojo.mes.bean.MesStation;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description : 站点上料扫描容器条码工步
 * @Author : wangjie
 **/
@Slf4j
@Service("mesStationFeedContainerSnStepService")
public class MesStationFeedContainerSnStepService extends BaseStepService {

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private IMesProductionCustomContextStepService productionCustomContextStepService;

    @Autowired
    private IMesStationContainerSnExtService stationContainerSnExtService;

    @Override
    public StepResult guide(StationRequestBean reqBean) {

        //发送工步内容
        productionCustomContextStepService.sendStepContextMessage(reqBean);

        return stepSuccessCompleteAndSendGuideReturn(reqBean, new StationResultBean().writeDbLog(MesPcnEnumUtil.WORK_CELL_SCAN_MONITOR_LOG_TYPE.PROCESS.getValue()), "请扫描容器条码!");

    }

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();

        //投料站点标识
        String flag = (String) reqBean.getDataMap().get(MesPcnExtConstWords.MATCH_STATION_CONTEXT_FLAG);

        if (StringUtils.isEmpty(flag)) stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "站点上料扫描容器条码时验证缺失必要flag参数!");

        if (StringUtils.isEmpty(reqBean.getScanInfo())) stepSendGuideAndThrowEx(reqBean, resultBean.writeDbLog(MesPcnEnumUtil.WORK_CELL_SCAN_MONITOR_LOG_TYPE.PROCESS.getValue()), "请扫描容器条码!");

        //获取站点用于缺料时进行上料绑定
        List<MesStation> stationList = productionDispatchContextStepService.getMatchStationContext(reqBean, flag);
        if (CollectionUtils.isEmpty(stationList)) stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "站点上料扫描容器条码时验证缺失站点信息!");

        //删除源工步传过来的参数标识
        reqBean.getDataMap().remove(MesPcnExtConstWords.MATCH_STATION_CONTEXT_FLAG);

        //默认取第一个站点
        MesStation station = stationList.stream().filter(o -> null != o).sorted(Comparator.comparing(MesStation::getStationType).thenComparing(MesStation::getCreateDatetime)).collect(Collectors.toList()).get(0);

        //验证当前需要绑定的箱的状态
        Integer packageStatus = MesExtEnumUtil.STATION_TYPE.getPackageStatus(station.getStationType());

        //获取容器条码上料主表信息【进料】的一条数据
        MesContainerPackage containerPackage = stationContainerSnExtService.getContainerPackageByContainerSn(reqBean.getOrganizeCode(), Stream.of(reqBean.getScanInfo()).collect(Collectors.toList()), null);

        //验证容器条码是否为已关箱状态
        if (null != packageStatus && packageStatus.compareTo(MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_20.getValue()) == 0
                && (null == containerPackage || containerPackage.getPackageStatus().compareTo(MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_10.getValue()) == 0)) {
            return stepNonCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, String.format("未查询到[%s]状态的容器条码[%s]!", MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_20.getDescription(), reqBean.getScanInfo()));
        }

        //验证容器条码是否为未关箱状态
        if (null != packageStatus && packageStatus.compareTo(MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_10.getValue()) == 0
                && null != containerPackage && containerPackage.getPackageStatus().compareTo(MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_20.getValue()) == 0) {
            return stepNonCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, String.format("未查询到[%s]状态的容器条码[%s]!", MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_10.getDescription(), reqBean.getScanInfo()));
        }

        //根据容器条码查询容器条码与站点的对应关系, 验证容器条码是否已经绑定过站点
        MesContainerSnStation containerSnStation = stationContainerSnExtService.getContainerSnStation(reqBean.getOrganizeCode(), reqBean.getScanInfo());
        if (null != containerSnStation) {
            return stepNonCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, String.format("容器条码[%s]已经被站点[%s]绑定!请重新扫描!", reqBean.getScanInfo(), containerSnStation.getStation()));
        }

        //验证容器 条码是否被套了大箱【大箱套小箱场景】
        MesContainerPackageDetail containerPackageDetail = stationContainerSnExtService.getContainerPackageDetailByBarCode(reqBean.getOrganizeCode(), reqBean.getScanInfo());
        MesContainerPackage containerPackage2Big = null == containerPackageDetail ? null : stationContainerSnExtService.getContainerPackageById(reqBean.getOrganizeCode(), containerPackageDetail.getPid());
        if (null != containerPackage2Big) return stepNonCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, String.format("容器条码[%s]已经被[大箱]容器条码[%s]绑定!请重新扫描!", reqBean.getScanInfo(), containerPackageDetail.getContainerSn()));

        //获取料桶容器条码 【递归获取大箱套小箱】
        List<MesContainerPackage> containerPackageList = new ArrayList<>();
        getContainerPackageList2Big(reqBean.getOrganizeCode(), containerPackage, containerPackageList);

        stationContainerSnExtService.insertContainerSnStationList(reqBean.getOrganizeCode(), reqBean.getUserInfo(), station.getStation(), containerPackageList, reqBean.getScanInfo());
        return stepSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, String.format("容器条码[%s]与站点[%s]绑定成功!", reqBean.getScanInfo(), stationList.get(0).getStation()));

    }

    //获取料桶容器条码 【递归获取大箱套小箱】
    private void getContainerPackageList2Big(String organizeCode, MesContainerPackage containerPackage, List<MesContainerPackage> targetList) {
        if (StringUtils.isEmpty(organizeCode) || null == containerPackage) return;
        List<MesContainerPackageDetail> containerPackageDetailList = stationContainerSnExtService.getContainerPackageDetailListByPid(organizeCode, containerPackage.getId());
        if (CollectionUtils.isEmpty(containerPackageDetailList) || StringUtils.isEmpty(containerPackageDetailList.get(0).getDataType())) return;
        if (containerPackageDetailList.get(0).getDataType().compareTo(MesExtEnumUtil.CONTAINER_BARCODE_DATA_TYPE.STATUS_30.getValue()) != 0) {
            targetList.add(containerPackage);
        } else {
            for (MesContainerPackageDetail containerPackageDetail : containerPackageDetailList) {
                MesContainerPackage containerPackageItem = stationContainerSnExtService.getContainerPackageById(organizeCode, containerPackageDetail.getPid());
                if (null == containerPackageItem)
                getContainerPackageList2Big(organizeCode, containerPackageItem, targetList);
            }
        }

    }

}
