package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step.method.stationpm;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.StringEqualsUtil;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesContainerPackageDetailContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesContainerPackageDetailStationContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesStation;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description : 加工模式 - 单件
 * @Author : wangjie
 **/
@Slf4j
@Service("mesStationMatchPsProcessMethodSingleStepService")
public class MesStationMatchPsProcessMethodSingleStepService extends BaseStepService implements IMesStationMatchPsProcessMethodStrategyStepService {

    @Autowired
    private IMesStationMatchPsProcessMethodStrategyStepService defaultDecoratorService;

    @Autowired
    private IMesProductionCustomContextStepService productionCustomContextStepService;

    //【原料站点】获取容器条码上料明细表信息
    @Override
    public MesContainerPackageDetailStationContext getContainerPackageDetailStationContext(String organizeCode, List<MesStation> stationList) {
        return getContainerPackageDetailStationContext(defaultDecoratorService, organizeCode, stationList);
    }

    //【原料站点】获取容器条码上料明细表信息
    @Override
    public MesContainerPackageDetailStationContext getContainerPackageDetailStationContext(IMesStationMatchPsProcessMethodStrategyStepService decoratorService, String organizeCode, List<MesStation> stationList) {
        MesContainerPackageDetailStationContext containerPackageDetailStationContext = null;
        for (MesStation station : stationList) {
            if (null == station) continue;
            MesContainerPackageDetailStationContext decoratorContext = decoratorService.getContainerPackageDetailStationContext(organizeCode, stationList);
            if (null == decoratorContext) continue;
            if (null == containerPackageDetailStationContext) containerPackageDetailStationContext = new MesContainerPackageDetailStationContext();
            containerPackageDetailStationContext.addStationList(decoratorContext.getStationList()).addStationContainerSnDetailList(decoratorContext);
        }
        return containerPackageDetailStationContext;
    }

    //【原料站点】获取进料主条码信息
    @Override
    public List<MesContainerPackageDetailContext> getContainerPackageDetailContext(StationRequestBean reqBean, MesContainerPackageDetailStationContext context, Integer cavity, Boolean isExistProductionPart) {
        List<MesContainerPackageDetailContext> resultList = null;
        if (null == context) return resultList;
        List<MesContainerPackageDetailStationContext> containerPackageDetailStationContextList = context.getStationContainerSnDetailList();
        for (MesContainerPackageDetailStationContext containerPackageDetailStationContext : containerPackageDetailStationContextList) {
            if (null == containerPackageDetailStationContext) continue;
            LinkedHashMap<String, List<MesContainerPackageDetailContext>> containerSnDataMap = containerPackageDetailStationContext.getTopContainerSnDetailMap2Sort();
            LOOP:
            for (Map.Entry<String, List<MesContainerPackageDetailContext>> entry : containerSnDataMap.entrySet()) {
                if (null == entry) continue;
                List<MesContainerPackageDetailContext> containerSnDetailList = entry.getValue();
                for (MesContainerPackageDetailContext containerPackageDetailContext : containerSnDetailList) {
                    //验证是否已经被使用
                    if (null == containerPackageDetailContext || !StringUtils.isEmpty(containerPackageDetailContext.getIsUsed())) continue;
                    //已经被匹配成功 matchResult不为空只会是在存在产成的情况下, 下面会先标记为0
                    if (!StringUtils.isEmpty(containerPackageDetailContext.getMatchResult()) && StringEqualsUtil.stringEquals(containerPackageDetailContext.getMatchResult(), CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValueStr())) continue;
                    //已经被匹配失败, 跳到下个站点进行验证第一个未匹配的进料主条码
                    if (!StringUtils.isEmpty(containerPackageDetailContext.getMatchResult()) && StringEqualsUtil.stringEquals(containerPackageDetailContext.getMatchResult(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValueStr())) break LOOP;
                    //验证是否被使用
                    if (productionCustomContextStepService.checkContainerPackageDetailPsLockedIsUsed(reqBean, MesPcnExtConstWords.PRODUCT_SN, containerPackageDetailContext.getBarCodeKey())) {
                        //标记被使用
                        containerPackageDetailContext.isUsed();
                        continue;
                    }

                    //如果存在产成, 标记即将进行匹配, 直接返回进行单腔验证, 支持跨站点按照站点内的容器条码顺序进行一个个匹配
                    if (isExistProductionPart) {
                        //一个个匹配才需要标记为0, 匹配完成之后需要将0的数据标记为1或者2
                        containerPackageDetailContext.matchResult(MesPcnExtConstWords.ZERO_STR);
                        return Stream.of(containerPackageDetailContext).collect(Collectors.toList());
                    }

                    //不存在产成, 按顺序添加进料主条码,直到满足设备腔数
                    if (CollectionUtils.isEmpty(resultList)) resultList = new ArrayList<>();
                    resultList.add(containerPackageDetailContext);
                    if (resultList.size() == cavity) return resultList;
                }
            }
        }
        return resultList;
    }

    //【原料站点】验证进料主条码个数是否满足腔数
    // count1=进料主条码个数   count2=加工规则对象个数   count3=产成零件个数[腔数]  cavity=设备腔数
    @Override
    public Boolean checkIsMatchCavity(Integer count1, Integer count2, Integer count3, Integer cavity, Boolean isBeforeCheck) {

        //前置验证
        if (isBeforeCheck) {
            //不存在产成, 需要一次取出满足设备腔数个数的进料, 验证进料是否满足设备腔数
            if (count3.compareTo(MesPcnExtConstWords.ZERO) == 0) return count1.compareTo(cavity) == 0;
            //存在产成, 进料等于0则是一定不够腔数的,因为后面满足腔数就进入下一个工步集了
            return count1.compareTo(MesPcnExtConstWords.ZERO) > 0;
        }

        //后置验证, 为true或者false 均退出循环
        //不存在产成, 验证加工规则是否满足设备腔数
        if (count3.compareTo(MesPcnExtConstWords.ZERO) == 0) return count2.compareTo(cavity) == 0;
        //存在产成, 验证加工规则是否满足产成腔数, 为null继续循环处理
        return count2.compareTo(count3) == 0 ? true : null;
    }

    //【原料站点】锁定进料主条码信息
    @Override
    public void dispatchContainerPackageDetailPsLocked(StationRequestBean reqBean, List<MesContainerPackageDetailContext> containerPackageDetailContextList) {
        containerPackageDetailContextList.forEach(o -> productionCustomContextStepService.dispatchContainerPackageDetailPsLocked(reqBean, MesPcnExtConstWords.PRODUCT_SN, o.getBarCodeKey()));
    }

    //【原料站点】解锁进料主条码信息
    @Override
    public void removeContainerPackageDetailPsLocked(StationRequestBean reqBean, List<MesContainerPackageDetailContext> containerPackageDetailContextList) {
        containerPackageDetailContextList.forEach(o -> productionCustomContextStepService.removeContainerPackageDetailPsLocked(reqBean, MesPcnExtConstWords.PRODUCT_SN, o.getBarCodeKey()));
    }

}
